@@ -224,6 +224,7 @@ class DataPanel(QtWidgets.QWidget, loadUiType(__file__.split(".py")[0] + ".ui")[
         self.overview.enableAutoRange(pg.ViewBox.XYAxes)
         self.xslice.enableAutoRange(pg.ViewBox.XYAxes)
         self.yslice.enableAutoRange(pg.ViewBox.XYAxes)
+        self._update_legend()
 
     def _update_legend(self):
         try:
@@ -296,12 +297,18 @@ class DataPanel(QtWidgets.QWidget, loadUiType(__file__.split(".py")[0] + ".ui")[
         self.selected_t_i = 0
         c_start = np.array(config.data["rawdata"]["scangradient"]["start"])
         c_stop = np.array(config.data["rawdata"]["scangradient"]["stop"])
+        # Include all scans if excluded scans not specified yet
+        if not "exclude_scans" in ds.t["raw"].attrs:
+            ds.t["raw"].attrs["exclude_scans"] = []
+        # Ensure any excluded scans are invisible
+        scan_enabled_list = np.ones((ds.d["raw/data"].shape[0],), dtype=np.bool)
+        scan_enabled_list[ds.t["raw"].attrs["exclude_scans"]] = False
         for i, scan_data in enumerate(ds.d["raw/data"]):
             # Compute colour in the gradient to use
             c = tuple((c_start + ((i/(ds.d["raw/data"].shape[0] - 1)) if ds.d["raw/data"].shape[0] > 1 else 0)*(c_stop - c_start)).astype(np.int))
             nonzero_t_i = np.any(scan_data, axis=1)
-            self.xslice_scan_plots.append(self.xslice.plot(ds.d["raw/time"][:][nonzero_t_i], scan_data[nonzero_t_i,self.selected_w_i], pen=c)) 
-            self.yslice_scan_plots.append(self.yslice.plot(ds.d["raw/wavelength"], scan_data[self.selected_t_i,:], pen=c))
+            self.xslice_scan_plots.append(self.xslice.plot(ds.d["raw/time"][:][nonzero_t_i], scan_data[nonzero_t_i,self.selected_w_i], pen=c if scan_enabled_list[i] else None)) 
+            self.yslice_scan_plots.append(self.yslice.plot(ds.d["raw/wavelength"], scan_data[self.selected_t_i,:], pen=c if scan_enabled_list[i] else None))
             # Store the assigned colour in the PlotItem so we can retrieve it later
             # This way we can toggle pen=None/pen=baseColor to hide an individual trace
             self.xslice_scan_plots[-1].baseColor = c
@@ -312,7 +319,9 @@ class DataPanel(QtWidgets.QWidget, loadUiType(__file__.split(".py")[0] + ".ui")[
             self.yslice_scan_plots[-1].sigClicked.connect(self.scansWidget.highlight_scan)
             self.yslice_scan_plots[-1].curve.setClickable(True)
         # Update the scan selection checkboxes
-        self.scansWidget.set_scans(zip(self.xslice_scan_plots, self.yslice_scan_plots))
+
+        
+        self.scansWidget.set_scans(list(zip(self.xslice_scan_plots, self.yslice_scan_plots)), checked=scan_enabled_list)
         
         # Update the average traces
         nonzero_t_i = np.any(ds.t["raw/data_avg"], axis=1)
@@ -471,12 +480,15 @@ class DataPanelScansWidget(QtWidgets.QFrame, loadUiType(os.path.join(os.path.dir
         self.highlighted_item = None
         self.set_scans(plotitems, label=label)
     
-    def set_scans(self, plotitems, label="Scan #{}"):
+    def set_scans(self, plotitems, checked=None, label="Scan #{}"):
         # Remove any existing checkboxes
         for checkbox in self.items.keys():
             self.scrollWidget.layout().removeWidget(checkbox)
             checkbox.setParent(None)
         self.items = {}
+        # All checkboxes are checked if not specified
+        if checked is None:
+            checked = [True]*len(plotitems)
         # Create new checkboxes for given PlotItems
         for i, plotitem in enumerate(plotitems):
             # Ensure plotitem is a tuple
@@ -485,7 +497,7 @@ class DataPanelScansWidget(QtWidgets.QFrame, loadUiType(os.path.join(os.path.dir
             except TypeError:
                 plotitem = (plotitem,)
             checkbox = QtWidgets.QCheckBox(label.format(i + 1))
-            checkbox.setChecked(True)
+            checkbox.setChecked(bool(checked[i]))
             checkbox.stateChanged.connect(lambda state, checkbox=checkbox: self._checkbox_clicked(state, checkbox))
             checkbox.installEventFilter(self)
             pen = plotitem[0].baseColor
@@ -503,7 +515,7 @@ class DataPanelScansWidget(QtWidgets.QFrame, loadUiType(os.path.join(os.path.dir
         for i, checkbox in enumerate(self.items.keys()):
             if not checkbox.isChecked():
                 excluded_scans.append(i)
-        ds.d["raw/data"].attrs["exclude_scans"] = excluded_scans
+        ds.t["raw"].attrs["exclude_scans"] = excluded_scans
         # Notify any listeners of the change to selection
         ds.signals.raw_selection_changed.emit()
 
