@@ -44,12 +44,15 @@ log = logging.getLogger(__name__)
 d = {}
 "Reference to the currently loaded data as a zarr array."
 
+d_path = ""
+"Path to the currently loaded zarr data array in :attr:`d`."
+
 t = zarr.group()
 "Reference to temporary data which doesn't need to be saved to disk or can be rebuilt from :attr:`d`."
 
 root_attrs = {
     "Description"      : "Time-resolved spectroscopy data",
-    "Format"           : "tadata",
+    "Format"           : "trdata",
     "Version"          : "1.0.0",
     "CreationSoftware" : "trspectrometer",
 }
@@ -124,20 +127,21 @@ def raw_data_average():
     """
     Compute and return the average of the raw data set.
 
-    This routine will use the "exclude_scans" attribute of raw/data to 
-    exclude scans from the average.
+    This routine will use the "exclude_scans" attribute of raw to exclude scans from the average.
     The attribute should contain a list of scan indices to exclude.
 
     :returns: Array containing the averaged data.
     """
-    if not "raw/data" in d: return None
-    if not "exclude_scans" in d["raw/data"].attrs:
+    if (not "raw" in d) or (not "raw/data" in d): return None
+    if not "raw" in t:
+        t.create_group("raw")
+    if not "exclude_scans" in t["raw"].attrs:
         # Include all scans if not specified yet
-        d["raw/data"].attrs["exclude_scans"] = []
-    if d["raw/data"].shape[0] - len(d["raw/data"].attrs["exclude_scans"]) >= 1:
+        t["raw"].attrs["exclude_scans"] = []
+    if d["raw/data"].shape[0] - len(t["raw"].attrs["exclude_scans"]) >= 1:
         # Multiple scans included, average them
         selection_mask = np.ones((d["raw/data"].shape[0]), dtype=np.bool)
-        selection_mask[d["raw/data"].attrs["exclude_scans"]] = False
+        selection_mask[t["raw"].attrs["exclude_scans"]] = False
         # Ignore warining if all NaNs in slice
         warnings.simplefilter("ignore")
         result = np.nanmean(d["raw/data"].get_orthogonal_selection((selection_mask, slice(None), slice(None))), axis=0)
@@ -154,7 +158,7 @@ def raw_data_average():
 
 def open_zarr(parent=None, initial_dir=None):
     """
-    Show a dialog for selecting a time-resolved data directory, and return the data.
+    Show a dialog for selecting a time-resolved data directory, and load the data.
 
     The native format for data is a Zarr data directory.
     If the selected directory does not appear to contain time-resolved data,
@@ -163,8 +167,9 @@ def open_zarr(parent=None, initial_dir=None):
     :param parent: The parent QWidget for the dialog box.
     :param initial_dir: Initial directory for the dialog box.
         If `None`, then the data directory specified in the configuration will be used.
-    :returns: Tuple of selected directory name and zarr data.
+    :returns: Name of selected data directory, or ``None`` if no directory selected.
     """
+    global d, t, d_path
     if initial_dir is None:
         initial_dir = config.data["directories"]["data"]
     # Was hoping that using a filter and directory options would limit to 
@@ -176,13 +181,22 @@ def open_zarr(parent=None, initial_dir=None):
     if qfd.exec_():
         dirname = qfd.selectedFiles()[0]
         try:
-            data = zarr.open(dirname, mode="r+")
+            # Open read-only to prevent inadvertant modifications to data
+            data = zarr.open(dirname, mode="r")
         except:
             raise RuntimeError(f"Unable to read {dirname}:\nNot a Zarr data directory.")
-        if not "Format" in data.attrs or not data.attrs["Format"] == "tadata":
+        if not "Format" in data.attrs or not data.attrs["Format"] == "trdata":
             raise RuntimeError(f"Unable to read {dirname}:\nNot time-resolved data.")
-        return dirname, data
-    return None, None
+        # Looks OK, switch in the loaded data
+        d = data
+        t = zarr.group()
+        # Copy data and metadata which may be modified to temporary storage
+        if ("raw" in d) and ("exclude_scans" in d["raw"].attrs):
+            t.create_group("raw")
+            t["raw"].attrs["exclude_scans"] = d["raw"].attrs["exclude_scans"]
+        d_path = dirname
+        return dirname
+    return None
 
 def save_zarr(parent=None, initial_dir=None, data=None):
     """
@@ -195,8 +209,10 @@ def save_zarr(parent=None, initial_dir=None, data=None):
     :param initial_dir: Initial directory for the dialog box.
         If `None`, then the data directory specified in the configuration will be used.
     :param data: Zarr data to save.
+    :returns: Directory where data was saved, or ``None`` if no directory selected.
     """
     # If data not given, default to currently loaded data
+    global d, t, d_path
     if data is None:
         if d is None: return None
         data = d
@@ -209,31 +225,53 @@ def save_zarr(parent=None, initial_dir=None, data=None):
     dirname = None
     while qfd.exec_():
         dirname = qfd.selectedFiles()[0]
-        # Annoyingly, QFileDialog creates the directory before the suffix is appended
-        # Delete it if empty
-        try:
-            os.rmdir(dirname)
-        except Exception as ex:
-            pass
-        # Ensure name has a .ta.zarr suffix
-        dirname = dirname.partition(".ta.zarr")[0] + ".ta.zarr"
-        try:
-            dest = zarr.open(dirname, mode="w-")
-        except Exception as ex:
-            reply = QMessageBox.question(parent, "Data Directory Exists", "Selected data directory already exists. Do you want to overwrite its contents?")
+        if dirname == d_path:
+            # Selected data dir is the one currently open
+            reply = QMessageBox.question(parent, "Data Directory Exists", "Selected data directory is the one currently open. Do you want to overwrite its contents?")
             if reply == QMessageBox.No:
                 # Show selection dialog again
-                initial_dir = dirname
                 dirname = None
                 continue
             else:
-                dest = zarr.open(dirname, mode="w")
-        try:
-            zarr.copy_all(data, dest)
-            # Root attributes don't get copied. Bug in zarr 2.5.0
-            dest.attrs.update(data.attrs)
-        except Exception as ex:
-            raise RuntimeError(f"Unable to write {dirname}:\n{ex}")
+                # Just need to update currently open data directory, enable write
+                dest = zarr.open(dirname, mode="r+")
+        else:
+            # Selected data dir is not the one currently open
+            # QFileDialog creates the directory before the suffix is appended, delete it if empty
+            try:
+                os.rmdir(dirname)
+            except Exception as ex:
+                pass
+            # Ensure name has a .tr.zarr suffix
+            dirname = dirname.partition(".tr.zarr")[0] + ".tr.zarr"
+            try:
+                dest = zarr.open(dirname, mode="w-")
+            except Exception as ex:
+                # Failed with "w-" mode as directory already exists
+                reply = QMessageBox.question(parent, "Data Directory Exists", "Selected data directory already exists. Do you want to overwrite its contents?")
+                if reply == QMessageBox.No:
+                    # Show selection dialog again
+                    initial_dir = dirname
+                    dirname = None
+                    continue
+                else:
+                    # Force overwrite of directory
+                    dest = zarr.open(dirname, mode="w")
+            # Copy the data over to new destination
+            try:
+                zarr.copy_all(data, dest)
+                # Root attributes don't get copied. Bug in zarr 2.5.0
+                dest.attrs.update(data.attrs)
+            except Exception as ex:
+                raise RuntimeError(f"Unable to write {dirname}:\n{ex}")
+        # Save out data modifications from temporary storage
+        if ("raw" in t) and ("exclude_scans" in t["raw"].attrs):
+            if "raw" not in dest:
+                dest.create_group("raw")
+            dest["raw"].attrs["exclude_scans"] = t["raw"].attrs["exclude_scans"]
+        # Done. Start using new data directory in read-only mode
+        d_path = dirname
+        d = zarr.open(dirname, "r")
         break
     return dirname
         
